/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_flash.c
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include "rx32s11xx_flash.h"
#include "rx32s11xx_cmu.h"

/*
*********************************************************************************************************
*                                             Local variable
*********************************************************************************************************
*/
#define flash_size 32

/*
*********************************************************************************************************
*                                            Local function
*********************************************************************************************************
*/

/**
*********************************************************************************************************
*@Function chip_erase
*@brief    Erase all FLASH addresses after the start address
*@retval None
*********************************************************************************************************
*/
void Flash_chip_erase(void)
{
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->FLASHLOCK = CMU_FLASHLOCK_UnLocked;
    CMU->FLASHCON = CMU_FLASHCON_FOP_CHIPERASE;
    M32(0x00000000) = 0xFF;
    while(CMU->FLASHCON & CMU_FLASHCON_BUSY);
    CMU->FLASHLOCK = CMU_FLASHLOCK_Locked;
    CMU->WPREG = CMU_WPREG_Protected;
}
/**
*********************************************************************************************************
*@Function Flash_page_erase
*@brief    Erase the specified page
*@param1   page:
*          @arg 6
*          @arg 7
*          @arg 8
*          @arg ...
*          @arg MAX = 31
*@retval   TRUE
*********************************************************************************************************
*/
Bool Flash_page_erase(uint8_t page)
{
    if (page >= flash_size)
    {
        return FALSE;
    }
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->FLASHLOCK = CMU_FLASHLOCK_UnLocked;
    CMU->FLASHCON = CMU_FLASHCON_FOP_PAGEERASE;
    M32(FLASH_BASE + 1024 * page) = 0xFF;
    while(CMU->FLASHCON & CMU_FLASHCON_BUSY);
    CMU->FLASHLOCK = CMU_FLASHLOCK_Locked;
    CMU->WPREG = CMU_WPREG_Protected;
    return TRUE;
}
/**
*********************************************************************************************************
*@Function Flash_Wordwrite
*@brief    write the buffer data to the specified address
*@param1   address:
*          @arg 6*1024
*          @arg 7*1024
*            ...
*          @arg 31*1024
*@param2   pwbuf:
*          @arg write_data
*
*@retval   TRUE
*********************************************************************************************************
*/
Bool Flash_Wordwrite(uint32_t address, uint32_t pwbuf)
{
    if (address >= FLASH_MAX_ADDRESS)
    {
        return FALSE;
    }
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->FLASHLOCK = CMU_FLASHLOCK_UnLocked;
    CMU->FLASHCON = CMU_FLASHCON_FOP_WRITE;
    M32(address) = pwbuf;
    while(CMU->FLASHCON & CMU_FLASHCON_BUSY);
    CMU->FLASHLOCK = CMU_FLASHLOCK_Locked;
    CMU->WPREG = CMU_WPREG_Protected;
    return TRUE;
}
/**
*********************************************************************************************************
*@Function Flash_HalfWordwrite
*@brief    write the buffer data to the specified address
*@param1   address:
*          @arg 6*1024
*          @arg 7*1024
*            ...
*          @arg 31*1024
*@param2   pwbuf:
*          @arg write_data
*
*@retval   TRUE
*********************************************************************************************************
*/
Bool Flash_HalfWordwrite(uint32_t address, uint16_t pwbuf)
{
    if (address >= FLASH_MAX_ADDRESS)
    {
        return FALSE;
    }
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->FLASHLOCK = CMU_FLASHLOCK_UnLocked;
    CMU->FLASHCON = CMU_FLASHCON_FOP_WRITE;

    M16(address) = pwbuf;
    while(CMU->FLASHCON & CMU_FLASHCON_BUSY);
    CMU->FLASHLOCK = CMU_FLASHLOCK_Locked;
    CMU->WPREG = CMU_WPREG_Protected;
    return TRUE;
}
/**
*********************************************************************************************************
*@Function Flash_Bytewrite
*@brief    write the buffer data to the specified address
*@param1   address:
*          @arg 6*1024
*          @arg 7*1024
*            ...
*          @arg 31*1024
*@param2   pwbuf:
*          @arg write_data
*@param3   num:
*          @arg size of write_data
*
*@retval   TRUE
*********************************************************************************************************
*/
Bool Flash_Bytewrite(uint32_t address, uint8_t *pwbuf, uint32_t num)
{
    uint32_t i;
    uint32_t prgAddr;
    if (address >= FLASH_MAX_ADDRESS)
    {
        return FALSE;
    }
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->FLASHLOCK = CMU_FLASHLOCK_UnLocked;
    CMU->FLASHCON = CMU_FLASHCON_FOP_WRITE;
    for (i = 0; i < num; i++)
    {
        prgAddr = address + i;
        if (prgAddr >= FLASH_MAX_ADDRESS)
        {
            return FALSE;
        }
        M8(prgAddr) = pwbuf[i];
        while(CMU->FLASHCON & CMU_FLASHCON_BUSY);
    }		
    CMU->FLASHLOCK = CMU_FLASHLOCK_Locked;
    CMU->WPREG = CMU_WPREG_Protected;
    return TRUE;
}
/**
*********************************************************************************************************
*@Function Flash_WordRead
*@brief    read the buffer data to the specified address
*@param1   pReadByte:
*          @arg pReadByte
*@param2   address:
*          @arg 6*1024
*          @arg 7*1024
*            ...
*          @arg 31*1024
*
*@retval   pReadByte
*********************************************************************************************************
*/
uint32_t Flash_WordRead( uint32_t Address)
{
	  uint32_t pReadByte;
    if (Address < FLASH_MAX_ADDRESS)
    {
        CMU->WPREG = CMU_WPREG_UnProtected;
				CMU->FLASHCON = CMU_FLASHCON_FOP_READ;
				pReadByte = M32(Address);
				CMU->WPREG = CMU_WPREG_Protected;
				return pReadByte;
    }
    else
    {
      return 0;
    }      
}

/**
*********************************************************************************************************
*@Function Flash_HalfWordRead
*@brief    read the buffer data to the specified address
*@param1   pReadByte:
*          @arg pReadByte
*@param2   address:
*          @arg 6*1024
*          @arg 7*1024
*            ...
*          @arg 31*1024
*
*@retval   pReadByte
*********************************************************************************************************
*/
uint16_t Flash_HalfWordRead( uint32_t Address)
{
	  uint16_t pReadByte;
    if (Address < FLASH_MAX_ADDRESS)
    {
        CMU->WPREG = CMU_WPREG_UnProtected;
				CMU->FLASHCON = CMU_FLASHCON_FOP_READ;
				pReadByte = M16(Address);
				CMU->WPREG = CMU_WPREG_Protected;
				return pReadByte;
    }   
    else
    {
      return 0;
    }
}

/**
*********************************************************************************************************
*@Function Flash_ByteRead
*@brief    read the buffer data to the specified address
*@param1   pReadByte:
*          @arg pReadByte
*@param2   address:
*          @arg 6*1024
*          @arg 7*1024
*            ...
*          @arg 31*1024
*@param3   num:
*          @arg size of read_data
*
*@retval   none
*********************************************************************************************************
*/
void Flash_ByteRead(uint8_t *pReadByte, uint32_t Address, uint32_t Num)
{
    /*  assert_param  */
    uint32_t i;
    if (Address >= FLASH_MAX_ADDRESS)
    {
        return;
    }
    CMU->WPREG = CMU_WPREG_UnProtected;
    CMU->FLASHCON = CMU_FLASHCON_FOP_READ;
    for (i = 0; i < Num; i++)
    {
        if ((Address + i) >= FLASH_MAX_ADDRESS)
        {
            return;
        }
        pReadByte[i] = M8(Address + i);
    }
    CMU->WPREG = CMU_WPREG_Protected;
}


