/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_iic_lib.c
* By           : RX_SD_Team
* Description  : Only rx32s11xx
*********************************************************************************************************
*/
#include "system_rx32s11xx.h"
#include "rx32s11xx_iic.h"
#include "rx32s11xx.h"

/**
  * @brief  Deinitializes the I2Cx peripheral registers to their default reset values.
  * @param  I2Cx: where x can be 0, 1 to select the 
  *         I2C peripheral.
  * @retval None
  */
void I2C_DeInit(I2C_TypeDef* I2Cx)
{
  I2Cx->I2CADR = 0;
  I2Cx->I2CCON = 0;
  I2Cx->I2CDAT = 0;
  I2Cx->I2CSTA = 0;
}
/**
  * @brief  Fills each I2C_InitStruct member with its default value.
  * @param  I2C_InitStruct: pointer to a I2C_InitTypeDef structure which will
  *         be initialized.
  * @retval None
  */
void I2C_StructInit(I2C_InitTypeDef *I2C_InitStruct)
{
  I2C_InitStruct->Ack = ENABLE; 
  I2C_InitStruct->ClockSpeed = 25000;
  I2C_InitStruct->Fsys = SystemCoreClock;
  I2C_InitStruct->GeneralCallMode = ENABLE;
  I2C_InitStruct->OwnAddress = 0x60;
}

/**
  * @function  I2C_Enable
  * @brief  Enables I2C.
  * @param1  I2Cx
  *          @arg I2C
  * @retval None
  */
void I2C_Enable(I2C_TypeDef *I2Cx)
{
	SET_BIT(I2Cx->I2CCON, I2C_I2CCON_EN);
}

/**
  * @function  I2C_Init
  * @brief  Initializes the I2Cx peripheral according to the specified parameters in the I2C_InitStruct.
  * @param1  I2Cx
  *          @arg I2C
  * @param2  I2C_InitStruct: Pointer to a I2C_InitTypeDef structure that contains the configuration information for the specified I2C peripheral.
  * 					@param Fsys;
	*									@arg This parameter must be set to the real system clock
	*
  * 					@param ClockSpeed
	*									@arg 0 ~ 400000
	*
  * 					@param OwnAddress
	*									@arg 0x00 ~ 0x7F
	*
  * 					@param GeneralCallMode
	*									@arg ENABLE
	*									@arg DISABLE
	*
  * 					@param FunctionalState Ack
	*									@arg ENABLE
	*									@arg DISABLE
  * @retval None
  */
void I2C_Init(I2C_TypeDef *I2Cx, I2C_InitTypeDef *I2C_InitStruct)
{
    uint16_t I2CCON_CR = 0;
    uint16_t temp = 0;

    /* Set I2Cx Own Address */
    I2Cx->I2CADR = I2C_InitStruct->OwnAddress << 1;

    /* Configure I2Cx Generalcall mode */
    I2Cx->I2CADR |= I2C_InitStruct->GeneralCallMode;

    /* Configure the I2Cx clock frequency */
    I2CCON_CR = I2C_InitStruct->Fsys / (I2C_InitStruct->ClockSpeed * 4) - 1;

    temp = I2Cx->I2CCON;
    temp = ((I2CCON_CR & 0x03F8) << 5) | ((I2CCON_CR & 0x0004) << 5) | (I2CCON_CR & 0x0003);

    /* Configure I2Cx acknowledgement */
    temp |= I2C_InitStruct->Ack << 2;

    I2Cx->I2CCON = temp;
}

/**
  * @function  I2C_Generate_Start
  * @brief  Generates I2Cx communication START condition.
  * @param1 I2Cx
            @arg I2C
  * @param2 NewState: new state of the I2C START condition generation.
            @arg This parameter can be: ENABLE or DISABLE.
  * @retval None
  */
void I2C_Generate_Start(I2C_TypeDef *I2Cx)
{
    /* Generate a START condition */
		I2Cx->I2CCON = (I2Cx->I2CCON & 0xFFC7) | I2C_I2CCON_Start;
}

/**
  * @function  I2C_Generate_Stop
  * @brief  Generates I2Cx communication STOP condition.
  * @param1 I2Cx
            @arg I2C
  * @retval None
  */
void I2C_Generate_Stop(I2C_TypeDef *I2Cx)
{
    /* Generate a STOP condition */
		I2Cx->I2CCON = (I2Cx->I2CCON & 0xFFC7) | I2C_I2CCON_Stop;
}

/**
  * @function  I2C_Send_Data
  * @brief  Sends a data byte through the I2Cx peripheral.
  * @param  I2Cx
            @arg I2C
  * @param  Data
            @arg Byte to be transmitted.
  * @retval None
  */
void I2C_Send_Data(I2C_TypeDef *I2Cx, uint8_t Data)
{
    /* Write in the I2CDAT register the data to be sent */
    I2Cx->I2CDAT = Data;

    /* Clear the Start&Stop&Interrupt Bits to send data*/
    I2Cx->I2CCON &= 0xFFC7;
}

/**
  * @function  I2C_Receive_Data
  * @brief  Returns the most recent received data by the I2Cx peripheral.
  * @param  I2Cx
            @arg I2C
  * @retval The value of the received data.
  */
uint8_t I2C_Receive_Data(I2C_TypeDef *I2Cx)
{
    /* Return the data in the I2CDAT register */
    return (uint8_t)(I2Cx->I2CDAT);
}

/**
  * @function  I2C_Get_Status
  * @brief  Returns the last I2Cx Status.
  * @param  I2Cx
            @arg I2C
  * @retval The last Status
  */
uint32_t I2C_Get_Status(I2C_TypeDef *I2Cx)
{
    /* Return I2C handle state */
    return I2Cx->I2CSTA;
}

/**
  * @function  I2C_Set_SSIA
  * @brief  Config the specified I2C START&STOP&SI&Acknowledge Bits and Clear the Interrupt Flag.
  * @param1  I2Cx
            @arg I2C
  * @param2  I2C_Start_Cmd: new state of the I2C START Bits.
            @arg This parameter can be one of the following values:
                            @ref I2C_Start_Set
                            @ref I2C_Start_Reset
  * @param3  I2C_Stop_Cmd: new state of the I2C STOP Bits.
            @arg This parameter can be one of the following values:
                            @ref I2C_Stop_Set
                            @ref I2C_Stop_Reset
  * @param4  I2C_SI_Cmd: new state of the I2C SI Bits.
            @arg This parameter can be one of the following values:
                            @ref I2C_SI_Set
                            @ref I2C_SI_Reset
  * @param5  I2C_Acknowledge_Cmd: new state of the I2C Acknowledge Bits.
            @arg This parameter can be one of the following values:
                            @ref I2C_Acknowledge_Set
                            @ref I2C_Acknowledge_Reset
  * @retval None
  */
void I2C_Set_SSIA(I2C_TypeDef *I2Cx, uint16_t I2C_Start_Cmd, uint16_t I2C_Stop_Cmd, uint16_t I2C_SI_Cmd, uint16_t I2C_Acknowledge_Cmd)
{
    I2Cx->I2CCON = (I2Cx->I2CCON & 0xFFC3) | I2C_Start_Cmd | I2C_Stop_Cmd | I2C_SI_Cmd | I2C_Acknowledge_Cmd;
}

/**
  * @function  I2C_Set_SSI
  * @brief  Config the specified I2C START&STOP&SI&Acknowledge Bits and Clear the Interrupt Flag.
  * @param1  I2Cx
            @arg I2C
  * @param2  I2C_Start_Cmd: new state of the I2C START Bits.
            @arg This parameter can be one of the following values:
                            @ref I2C_Start_Set
                            @ref I2C_Start_Reset
  * @param3  I2C_Stop_Cmd: new state of the I2C STOP Bits.
            @arg This parameter can be one of the following values:
                            @ref I2C_Stop_Set
                            @ref I2C_Stop_Reset
  * @param4  I2C_SI_Cmd: new state of the I2C SI Bits.
            @arg This parameter can be one of the following values:
                            @ref I2C_SI_Set
                            @ref I2C_SI_Reset
  * @retval None
  */
void I2C_Set_SSI(I2C_TypeDef *I2Cx, uint16_t I2C_Start_Cmd, uint16_t I2C_Stop_Cmd, uint16_t I2C_SI_Cmd)
{
    I2Cx->I2CCON = (I2Cx->I2CCON & 0xFFC7) | I2C_Start_Cmd | I2C_Stop_Cmd | I2C_SI_Cmd;
}

/**
  * @function  I2C_Enable_Acknowledge
  * @brief  Enable the specified I2C Acknowledge feature.
  * @param  I2Cx
            @arg I2C
  * @retval None
  */
void I2C_Enable_Acknowledge(I2C_TypeDef *I2Cx)
{
	SET_BIT(I2Cx->I2CCON, I2C_I2CCON_Ack);
}

/**
  * @function  I2C_Enable_Acknowledge
  * @brief  Disable the specified I2C Acknowledge feature.
  * @param  I2Cx
            @arg I2C
  * @retval None
  */
void I2C_Disable_Acknowledge(I2C_TypeDef *I2Cx)
{
	CLEAR_BIT(I2Cx->I2CCON, I2C_I2CCON_Ack);
}

/**
  * @function  I2C_Enable_GeneralCall
  * @brief  Enable the specified I2C general call feature.
  * @param  I2Cx
            @arg I2C
  * @retval None
  */
void I2C_Enable_GeneralCall(I2C_TypeDef *I2Cx)
{
    SET_BIT(I2Cx->I2CADR, I2C_I2CADR_GeneralCall);
}

/**
  * @function  I2C_Disable_GeneralCall
  * @brief  Disable the specified I2C general call feature.
  * @param  I2Cx
            @arg I2C
  * @retval None
  */
void I2C_Disable_GeneralCall(I2C_TypeDef *I2Cx)
{
    CLEAR_BIT(I2Cx->I2CADR, I2C_I2CADR_GeneralCall);
}

/**
  * @function  I2C_Master_Transmit
  * @brief  Transmits in master mode an amount of data in blocking mode.
  * @param  I2Cx
  *          @arg I2C
  *	@param  I2Cx_Handle	:	Pointer to a I2C_HandleTypeDef structure that contains the configuration information for the specified I2C.
  * 					@param DevAddress;
	*									@arg 0x00 ~ 0x7F
  * 					@param MemAddress
	*									@arg 0x0000 ~ 0xFFFF
  * 					@param MemAddressSize
	*									@arg 8 or 16
  * 					@param pData
	*									@arg Pointer to I2C Data buffer
  * 					@param Size
	*									@arg I2C transfer size
  * @retval None
  */
void I2C_Master_Transmit(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle)
{
    uint32_t i = 0;
    uint8_t *pBuffPtr;
    pBuffPtr = I2Cx_Handle->pData;

    /* Generate a START condition */
    I2C_Generate_Start(I2Cx);
    /* Check a START condition has been send */
    while (I2C_Get_Status(I2Cx) != 0x0008)
        ;

    /* Send Slave Device Address */
    I2C_Send_Data(I2Cx, I2Cx_Handle->DevAddress << 1);
    /* Check the SLA+W has been send and ACK has been received */
    while (I2C_Get_Status(I2Cx) != 0x0018)
        ;

    /* Send Data buffer */
    for (i = 0; i < I2Cx_Handle->Size; i++)
    {
        /* Sends a data byte */
        I2C_Send_Data(I2Cx, *pBuffPtr);
        /* Check a data byte has been send and ACK has been received */
        while (I2C_Get_Status(I2Cx) != 0x0028)
            ;
        pBuffPtr++;
    }

    /* Generate a STOP condition */
    I2C_Generate_Stop(I2Cx);
}

/**
  * @function  I2C_Master_Receive
  * @brief  Receives in master mode an amount of data in blocking mode.
  * @param  I2Cx
  *          @arg I2C Instance
  *	@param	I2Cx_Handle	:	I2C_HandleTypeDef
  * 					@param DevAddress;
	*									@arg 0x00 ~ 0x7F
  * 					@param MemAddress
	*									@arg 0x0000 ~ 0xFFFF
  * 					@param MemAddressSize
	*									@arg 8 or 16
  * 					@param pData
	*									@arg Pointer to I2C Data buffer
  * 					@param Size
	*									@arg I2C transfer size
  * @retval None
  */
void I2C_Master_Receive(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle)
{
    uint16_t i = 0;
    uint8_t *pBuffPtr;
    pBuffPtr = I2Cx_Handle->pData;

    /* Enable the acknowledgement */
    I2C_Enable_Acknowledge(I2Cx);

    /* Generate a START condition */
    I2C_Generate_Start(I2Cx);
    /* Check a START condition has been send */
    while (I2C_Get_Status(I2Cx) != 0x0008)
        ;

    /* Send Slave Device Address */
    I2C_Send_Data(I2Cx, (I2Cx_Handle->DevAddress << 1) | 0x01);
    /* Check the SLA+R has been send and ACK has been received */
    while (I2C_Get_Status(I2Cx) != 0x0040)
        ;

    /* Read Data to Data buffer */
    for (i = 0; i < I2Cx_Handle->Size; i++)
    {
        if (i >= I2Cx_Handle->Size - 1)
        {
            /* Disable the acknowledgement */
            I2C_Disable_Acknowledge(I2Cx);
        }

        /* Clear the Start&Stop&Interrupt Bits */
        I2C_Set_SSI(I2Cx, I2C_Start_Reset, I2C_Stop_Reset, I2C_SI_Reset);
        if (i >= I2Cx_Handle->Size - 1)
        {
            /* Check a data byte has been received and NACK has been send */
            while (I2C_Get_Status(I2Cx) != 0x0058)
                ;
        }
        else
        {
            /* Check a data byte has been received and ACK has been send */
            while (I2C_Get_Status(I2Cx) != 0x0050)
                ;
        }
        /* Return the data in the I2CDAT register */
        *pBuffPtr = I2C_Receive_Data(I2Cx);
        pBuffPtr++;
    }

    /* Generate a STOP condition */
    I2C_Generate_Stop(I2Cx);
}

/**
  * @function  I2C_Slave_Transmit
  * @brief  Transmits in slave mode an amount of data in blocking mode.
  * @param  I2Cx
  *          @arg I2C
  *	@param	I2Cx_Handle	:	Pointer to a I2C_HandleTypeDef structure that contains the configuration information for the specified I2C.
  * 					@param DevAddress;
	*									@arg 0x00 ~ 0x7F
  * 					@param MemAddress
	*									@arg 0x0000 ~ 0xFFFF
  * 					@param MemAddressSize
	*									@arg 8 or 16
  * 					@param pData
	*									@arg Pointer to I2C Data buffer
  * 					@param Size
	*									@arg I2C transfer size
  * @retval None
  */
void I2C_Slave_Transmit(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle)
{
    uint16_t i = 0;
    uint8_t *pBuffPtr;
    pBuffPtr = I2Cx_Handle->pData;

    /* Enable the acknowledgement */
    I2C_Enable_Acknowledge(I2Cx);

    /* Check the SLA+R has been received and ACK has been send */
    while (I2C_Get_Status(I2Cx) != 0x00A8)
        ;

    /* Send Data buffer */
    for (i = 0; i < I2Cx_Handle->Size; i++)
    {
        if (i >= I2Cx_Handle->Size - 1)
        {
            /* Disable the acknowledgement */
            I2C_Disable_Acknowledge(I2Cx);
        }
        /* Sends a data byte */
        I2C_Send_Data(I2Cx, *pBuffPtr);
        if (i >= I2Cx_Handle->Size - 1)
        {
            /* Check a data byte has been send and NACK has been received */
            while (I2C_Get_Status(I2Cx) != 0x00C0)
                ;
        }
        else
        {
            /* Check a data byte has been send and ACK has been received */
            while (I2C_Get_Status(I2Cx) != 0x00B8)
                ;
        }
        pBuffPtr++;
    }

    /*Config I2Cx in unaddressed slave mode and do not identify the slave address and general call address*/
		I2C_Set_SSIA(I2Cx, I2C_Start_Reset, I2C_Stop_Reset, I2C_SI_Reset, I2C_Acknowledge_Reset);
}

/**
  * @function  I2C_Slave_Receive
  * @brief  Receive in slave mode an amount of data in blocking mode.
  * @param  I2Cx
  *          @arg I2C
  *	@param	I2Cx_Handle	:	Pointer to a I2C_HandleTypeDef structure that contains the configuration information for the specified I2C.
  * 					@param DevAddress;
	*									@arg 0x00 ~ 0x7F
  * 					@param MemAddress
	*									@arg 0x0000 ~ 0xFFFF
  * 					@param MemAddressSize
	*									@arg 8 or 16
  * 					@param pData
	*									@arg Pointer to I2C Data buffer
  * 					@param Size
	*									@arg I2C transfer size
  * @retval None
  */
void I2C_Slave_Receive(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle)
{
    uint16_t i = 0;
    uint8_t *pBuffPtr;
    pBuffPtr = I2Cx_Handle->pData;

    /* Enable the acknowledgement */
    I2C_Enable_Acknowledge(I2Cx);

    /* Check the SLA+W or Gnerall call address had been received and the ACK had been send */
    while ((I2C_Get_Status(I2Cx) != 0x0060) && (I2C_Get_Status(I2Cx) != 0x0070))
        ;

    for (i = 0; i < I2Cx_Handle->Size; i++)
    {
        /* Clear the Start&Stop&Interrupt Bits */
        I2C_Set_SSI(I2Cx, I2C_Start_Reset, I2C_Stop_Reset, I2C_SI_Reset);
        /* Check a data byte had been received and the ACK had been send */
        while ((I2C_Get_Status(I2Cx) != 0x0080) && (I2C_Get_Status(I2Cx) != 0x0090))
            ;
        /* Return the data in the I2CDAT register */
        *pBuffPtr = I2C_Receive_Data(I2Cx);
        pBuffPtr++;
    }

    /* Clear the Start&Stop&Interrupt Bits */
    I2C_Set_SSI(I2Cx, I2C_Start_Reset, I2C_Stop_Reset, I2C_SI_Reset);
    /* Wait until STOP condition has been received */
    while (I2C_Get_Status(I2Cx) != 0x00A0)
        ;
    /*Config I2Cx in unaddressed slave mode and do not identify the slave address and general call address*/
    I2C_Set_SSIA(I2Cx, I2C_Start_Reset, I2C_Stop_Reset, I2C_SI_Reset, I2C_Acknowledge_Reset);
}

/**
  * @function  I2C_Mem_Master_Write
  * @brief  Write an amount of data in blocking mode to a specific memory address.
  * @param  I2Cx
  *          @arg I2C
  *	@param	I2Cx_Handle	:	Pointer to a I2C_HandleTypeDef structure that contains the configuration information for the specified I2C.
  * 					@param DevAddress;
	*									@arg 0x00 ~ 0x7F
  * 					@param MemAddress
	*									@arg 0x0000 ~ 0xFFFF
  * 					@param MemAddressSize
	*									@arg 8 or 16
  * 					@param pData
	*									@arg Pointer to I2C Data buffer
  * 					@param Size
	*									@arg I2C transfer size
  * @retval None
  */
void I2C_Mem_Master_Write(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle)
{
    uint16_t i = 0;
    uint8_t *pBuffPtr;
    pBuffPtr = I2Cx_Handle->pData;

    /* Generate a START condition */
    I2C_Generate_Start(I2Cx);
    /* Check a START condition has been send */
    while (I2C_Get_Status(I2Cx) != 0x0008)
        ;

    /* Send Slave Device Address */
    I2C_Send_Data(I2Cx, I2Cx_Handle->DevAddress << 1);
    /* Check the SLA+W has been send and ACK has been received */
    while (I2C_Get_Status(I2Cx) != 0x0018)
        ;

    for (i = I2Cx_Handle->MemAddressSize; i > 0; i -= 8)
    {
        /* Send Slave Memory Address */
        I2C_Send_Data(I2Cx, (uint8_t)(I2Cx_Handle->MemAddress >> (i - 8)));
        /* Check the Memory Address has been send and ACK has been received */
        while (I2C_Get_Status(I2Cx) != 0x0028)
            ;
    }

    /* Send Data buffer */
    for (i = 0; i < I2Cx_Handle->Size; i++)
    {
        /* Sends a data byte */
        I2C_Send_Data(I2Cx, *pBuffPtr);
        /* Check a data byte has been send and ACK has been received */
        while (I2C_Get_Status(I2Cx) != 0x0028)
            ;
        pBuffPtr++;
    }

    /* Generate a STOP condition */
    I2C_Generate_Stop(I2Cx);
}

/**
  * @function  I2C_Mem_Master_Read
  * @brief  Read an amount of data in blocking mode from a specific memory address.
  * @param  I2Cx
  *          @arg I2C Instance
  *	@param	I2Cx_Handle	:	I2C_HandleTypeDef
  * 					@param DevAddress;
	*									@arg 0x00 ~ 0x7F
  * 					@param MemAddress
	*									@arg 0x0000 ~ 0xFFFF
  * 					@param MemAddressSize
	*									@arg 8 or 16
  * 					@param pData
	*									@arg Pointer to I2C Data buffer
  * 					@param Size
	*									@arg I2C transfer size
  * @retval None
  */
void I2C_Mem_Master_Read(I2C_TypeDef *I2Cx, I2C_HandleTypeDef *I2Cx_Handle)
{
    uint16_t i = 0;
    uint8_t *pBuffPtr;
    pBuffPtr = I2Cx_Handle->pData;

    /* Enable the acknowledgement */
    I2C_Enable_Acknowledge(I2Cx);

    /* Generate a START condition */
    I2C_Generate_Start(I2Cx);
    /* Check a START condition has been send */
    while (I2C_Get_Status(I2Cx) != 0x0008)
        ;

    /* Send Slave Device Address */
    I2C_Send_Data(I2Cx, I2Cx_Handle->DevAddress << 1);
    /* Check the SLA+W has been send and ACK has been received */
    while (I2C_Get_Status(I2Cx) != 0x0018)
        ;

    for (i = I2Cx_Handle->MemAddressSize; i > 0; i -= 8)
    {
        /* Send Slave Memory Address */
        I2C_Send_Data(I2Cx, (uint8_t)(I2Cx_Handle->MemAddress >> (i - 8)));
        /* Check the Memory Address has been send and ACK has been received */
        while (I2C_Get_Status(I2Cx) != 0x0028)
            ;
    }

    /* Generate a RESTART condition */
    I2C_Generate_Start(I2Cx);
    /* Check a RESTART condition has been send */
    while (I2C_Get_Status(I2Cx) != 0x0010)
        ;

    /* Send Slave Device Address */
    I2C_Send_Data(I2Cx, (I2Cx_Handle->DevAddress << 1) | 0x01);
    /* Check the SLA+R has been send and ACK has been received */
    while (I2C_Get_Status(I2Cx) != 0x0040)
        ;

    /* Read Data to Data buffer */
    for (i = 0; i < I2Cx_Handle->Size; i++)
    {
        if (i >= I2Cx_Handle->Size - 1)
        {
            /* Disable the acknowledgement */
            I2C_Disable_Acknowledge(I2Cx);
        }
        /* Clear the Start&Stop&Interrupt Bits */
        I2C_Set_SSI(I2Cx, I2C_Start_Reset, I2C_Stop_Reset, I2C_SI_Reset);
        if (i >= I2Cx_Handle->Size - 1)
        {
            /* Check a data byte has been received and ACK has been send */
            while (I2C_Get_Status(I2Cx) != 0x0058)
                ;
        }
        else
        {
            /* Check a data byte has been received and NACK has been send */
            while (I2C_Get_Status(I2Cx) != 0x0050)
                ;
        }
        /* Return the data in the I2CDAT register */
        *pBuffPtr = I2C_Receive_Data(I2Cx);
        pBuffPtr++;
    }

    /* Generate a STOP condition */
    I2C_Generate_Stop(I2Cx);
}
