/*********************************************************************************************
Copyright <2021> <Icore Technology (Nanjing)  Co.,Ltd>
All Rights Reserved,
Redistribution and use in source and binary forms, with or without modification, are permitted
provided that the following conditions are met:
1. Redistributions of source code must retain the above copyright notice, this list of
conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice, this list of
conditions and the following disclaimer in the documentation and/or other materials provided 
with the distribution.
3. Neither the name of the copyright holder nor the names of its contributors may be used to 
endorse or promote products derived from this software without specific prior written
permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
**********************************************************************************************/
/*
*********************************************************************************************************
*                                              rx32s11xx
*                                           Library Function
*
*                                        All Rights Reserved
*
*
* Project      : rx32s11xx
* File         : rx32s11xx_pmu.c
* By           : RX_SD_Team
* Version      : 20210420
* Description  : Only rx32s11xx
*********************************************************************************************************
*/

#include "rx32s11xx.h"
#include "rx32s11xx_pmu.h"
#include "rx32s11xx_cmu.h"
#include "rx32s11xx_cmp.h"
#include "rx32s11xx_adc.h"


/**
  * @brief  Deinitializes the PMUx peripheral registers to their default reset values.
  * @param  PMUx: where x can be 0, 1 to select the 
  *         PMU peripheral.
  * @retval None
  */
void PMU_DeInit(PMU_TypeDef* PMUx)
{
  PMUx->PMUCON = 0;
  PMUx->PMUIE = 0;
  PMUx->PMUIF = 0;
  PMUx->PMUSTA = 0;
  PMUx->RSTSTA = 0;
  PMUx->VDETCFG = 0;
  PMUx->VDETPCFG = 0;
  PMUx->WAKEIF = 0;
}

/**
*********************************************************************************************************
*@function    PMU_Set_LDOSLEEP
*@brief         Set LDO in SLEEP mode.
*                  The users can close the high power LDO,when the lowest power consumption is needed  in SLEEP mode.
*                  Chip can automatically switches low power LDO
*@param       ldo:
*                  @arg PMU_LDO_ENABLE
*                  @arg PMU_LDO_DISABLE
* @retval      None
*********************************************************************************************************
**/
void PMU_Set_LDOSLEEP(uint16_t ldo)
{
    MODIFY_REG(PMU->PMUCON, (PMU_READ_LDO << PMU_LDO_SHIFT), (ldo << PMU_LDO_SHIFT));
}

/**
*********************************************************************************************************
*@function    PMU_Get_LDOSLEEP
*@brief         Get LDO in SLEEP.
*@param       None
*@retval State of bit (1 or 0).
*********************************************************************************************************
**/
uint32_t PMU_Get_LDOSLEEP(void)
{
    return (READ_BIT(PMU->PMUCON, (PMU_READ_LDO << PMU_LDO_SHIFT)) >> PMU_LDO_SHIFT);
}

/**
*********************************************************************************************************
*@function    PMU_Set_VCCLVL
*@brief         Set VCC_DET detection threshold
*@param       vcclvl:
*@retval       @arg PMU_VCCLVL_2_2_v        ((uint32_t)0x0000)
*              @arg PMU_VCCLVL_2_8_v        ((uint32_t)0x0001)
*              @arg PMU_VCCLVL_3_6_v        ((uint32_t)0x0002)
*              @arg PMU_VCCLVL_4_2_v        ((uint32_t)0x0003)
* @retval      None
*********************************************************************************************************
*/
void PMU_Set_VCCLVL(uint16_t vcclvl)
{
    MODIFY_REG(PMU->VDETCFG, (PMU_READ_VCCLVL << PMU_VCCLVL_SHIFT), (vcclvl << PMU_VCCLVL_SHIFT));
}

/**
*********************************************************************************************************
*@function    PMU_Get_VCCLVL
*@brief         Get VCC_DET detection threshold
*@param       None
*@retval       @arg PMU_VCCLVL_2_2_v        ((uint32_t)0x0000)
*              @arg PMU_VCCLVL_2_8_v        ((uint32_t)0x0001)
*              @arg PMU_VCCLVL_3_6_v        ((uint32_t)0x0002)
*              @arg PMU_VCCLVL_4_2_v        ((uint32_t)0x0003)
*********************************************************************************************************
*/
uint32_t PMU_Get_VCCLVL(void)
{
    return (READ_BIT(PMU->VDETCFG, (PMU_READ_VCCLVL << PMU_VCCLVL_SHIFT)) >> PMU_VCCLVL_SHIFT);
}

/*
*********************************************************************************************************
*@function    PMU_Set_TimeSharingDetection
*@brief         VCC_DET & BOR_DET time-sharing detect in SLEEP & HOLD. The maximum detection time has been locked internally
*@param       time:
*                  @arg PMU_TIME_300_us
*                  @arg PMU_TIME_360_us
*                  @arg PMU_TIME_480_us
*                  @arg PMU_TIME_1120_us
*@param       period:
*                  @arg PMU_PERIOD_16_5_ms
*                  @arg PMU_PERIOD_33_ms
*                  @arg PMU_PERIOD_67_ms
*                  @arg PMU_PERIOD_134_ms
*                  @arg PMU_PERIOD_268_ms
*                  @arg PMU_PERIOD_536_ms
*                  @arg PMU_PERIOD_1072_ms
*                  @arg PMU_PERIOD_2144_ms
* @retval      None
*********************************************************************************************************
*/
void PMU_Set_TimeSharingDetection(uint16_t time, uint16_t period)
{
    MODIFY_REG(PMU->VDETPCFG, ((PMU_READ_TIME << PMU_TIME_SHIFT) | (PMU_READ_PERIOD << PMU_PERIOD_SHIFT)), ((time << PMU_TIME_SHIFT) | (period << PMU_PERIOD_SHIFT)));
}

/*
*********************************************************************************************************
*@function    PMU_Get_Time
*@brief         Get time-sharing detection time
*@param       None
* @retval      @arg PMU_TIME_300_us
*                  @arg PMU_TIME_360_us
*                  @arg PMU_TIME_480_us
*                  @arg PMU_TIME_1120_us
*********************************************************************************************************
*/
uint32_t PMU_Get_Time(void)
{
    return (READ_BIT(PMU->VDETPCFG, (PMU_READ_TIME << PMU_TIME_SHIFT)) >> PMU_TIME_SHIFT);
}

/*
*********************************************************************************************************
*@function    PMU_Get_Period
*@brief         Get time-sharing detection period
*@param       None
* @retval      @arg PMU_PERIOD_16_5_ms
*                  @arg PMU_PERIOD_33_ms
*                  @arg PMU_PERIOD_67_ms
*                  @arg PMU_PERIOD_134_ms
*                  @arg PMU_PERIOD_268_ms
*                  @arg PMU_PERIOD_536_ms
*                  @arg PMU_PERIOD_1072_ms
*                  @arg PMU_PERIOD_2144_ms
*********************************************************************************************************
*/
uint32_t PMU_Get_Period(void)
{
    return (READ_BIT(PMU->VDETPCFG, (PMU_READ_PERIOD << PMU_PERIOD_SHIFT)) >> PMU_PERIOD_SHIFT);
}

/*
*********************************************************************************************************
*@function    PMU_Enable_VCCIT
*@brief         Enable VCC_DET interrupt .
*@param       None
*@retval       None
*********************************************************************************************************
*/
void PMU_Enable_VCCIT(void)
{
    SET_BIT(PMU->PMUIE, (PMU_ENABLE_VCC_DET_IT << PMU_VCC_DET_IT_SHIFT));
}

/*
*********************************************************************************************************
*@function    PMU_Disable_VCCIT
*@brief         Disable VCC_DET interrupt .
*@param       None
*@retval       None
*********************************************************************************************************
*/
void PMU_Disable_VCCIT(void)
{
    CLEAR_BIT(PMU->PMUIE, (PMU_ENABLE_VCC_DET_IT << PMU_VCC_DET_IT_SHIFT));
}

/*
*********************************************************************************************************
*@function    PMU_Get_VCCIT
*@brief         Indicates whether the VCC_DET interrupt  is enabled.
*@param       None
*@retval       State of bit (1 or 0).
*********************************************************************************************************
*/
uint32_t PMU_Get_VCCIT(void)
{
    return (READ_BIT(PMU->PMUIE, (PMU_READ_VCC_DET_IT << PMU_VCC_DET_IT_SHIFT)) == (PMU_READ_VCC_DET_IT << PMU_VCC_DET_IT_SHIFT));
}

/*
*********************************************************************************************************
*@function    PMU_Get_VCCIF
*@brief         Indicate whether VCC_DET interrupt flag is set .
*@param       None
*@retval       State of bit (1 or 0).
*********************************************************************************************************
*/
uint32_t PMU_Get_FlagVCC(void)
{
    return (READ_BIT(PMU->PMUIF, (PMU_READ_VCC_DET_IF << PMU_VCC_DET_IF_SHIFT)) == (PMU_READ_VCC_DET_IF << PMU_VCC_DET_IF_SHIFT));
}

/*
*********************************************************************************************************
*@function    PMU_Clear_VCCIF
*@brief         Clear the VCC_DET interrupt flag.
*@param       None
*@retval       None
*********************************************************************************************************
*/
void PMU_Clear_FlagVCC(void)
{
    CLEAR_BIT(PMU->PMUIF, (PMU_READ_VCC_DET_IF << PMU_VCC_DET_IF_SHIFT));
}

/*
*********************************************************************************************************
*@function    PMU_Get_VCCState
*@brief         Get VCC state.Determine whether VCC is greater or less than VCC_LVL[3:0].
*@param       None
*@retval       @arg PMU_VCC_GREATER_VCCLVL
*              @arg PMU_VCC_LESS_VCCLVL
*********************************************************************************************************
*/
uint32_t PMU_Get_VCCState(void)
{
    return (READ_BIT(PMU->PMUSTA, (PMU_READ_VCC_STATE << PMU_VCC_STATE_SHIFT)) >> PMU_VCC_STATE_SHIFT);
}

/*
*********************************************************************************************************
*@function    PMU_Set_PDLDO
*@brief       Set PMU PDLDO.
*@param       status
*             @ref    PMU_PDLDO_ENABLE              ((uint32_t)0x0000)
*             @ref    PMU_PDLDO_DISABLE             ((uint32_t)0x0001)
*@retval      None
*********************************************************************************************************
*/
void PMU_Set_PDLDO(uint32_t status)
{
  MODIFY_REG(PMU->PMUCON,(PMU_READ_PDLDO<<PMU_PDLDO_SHIFT),(status<<PMU_PDLDO_SHIFT));
}

/*
*********************************************************************************************************
*@function    PMU_Get_WakeupFlag
*@brief         Get wake up flag.
*@param       wakeflag:
*                       @arg PMU_WAKEIF_PMUWKIF
*                       @arg PMU_WAKEIF_INT0WKIF
*                       @arg PMU_WAKEIF_INT1WKIF
*                       @arg PMU_WAKEIF_INT2WKIF
*                       @arg PMU_WAKEIF_INT3WKIF
*                       @arg PMU_WAKEIF_INT4WKIF
*                       @arg PMU_WAKEIF_INT5WKIF
*                       @arg PMU_WAKEIF_INT6WKIF
*                       @arg PMU_WAKEIF_RX0WKIF
*                       @arg PMU_WAKEIF_RX1WKIF
*                       @arg PMU_WAKEIF_RTCWKIF
*                       @arg PMU_WAKEIF_INT7WKIF
*                       @arg PMU_WAKEIF_INT8WKIF
*                       @arg PMU_WAKEIF_INT9WKIF
*@retval       State of bit (1 or 0).
*********************************************************************************************************
*/
uint32_t PMU_Get_WakeupFlag(uint32_t wakeflag)
{
    return (READ_BIT(PMU->WAKEIF, wakeflag) == (wakeflag));
}

/*
*********************************************************************************************************
*@function    EnterSleep1
*@brief       Enter sleep1 mode.
*@param       None
*@retval      None
*********************************************************************************************************
*/
void Enter_Sleep1(void)
{	
	//------------------------- SYS = LRC
	CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_LRC);
		//------------------------- disable HRC
	CMU_Disable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
  
	//HIIPM:OFF
	GPIOA->HIIPM = 0x0000;
	GPIOB->HIIPM = 0x0000;
	GPIOC->HIIPM = 0x0000;
	GPIOD->HIIPM = 0x0000;
	//Mode:GPIO
	GPIOA->IOCFG = 0x0000;
	GPIOB->IOCFG = 0x0000;
	GPIOC->IOCFG = 0x0000;
	GPIOD->IOCFG = 0x0000;
	
	//function:input
	GPIOA->PTDIR = 0x0000;
	GPIOB->PTDIR = 0x0000;
	GPIOC->PTDIR = 0x0000;
	GPIOD->PTDIR = 0x0000;
	
	//PUPDN:PULLUP
	GPIOA->PUPDN = 0xFFFF0000;
	GPIOB->PUPDN = 0xFFFF0000;
	GPIOC->PUPDN = 0xFFFF0000;
	GPIOD->PUPDN = 0xFFFF0000;
  
	PMU->PMUCON |= (1<<4);
	SCB->SCR = 0x0000;
	__WFI();
}

/*
*********************************************************************************************************
*@function    EnterSleep0
*@brief       Enter sleep0 mode.
*@param       None
*@retval      None
*********************************************************************************************************
*/
void Enter_Sleep0(void)
{	
	//------------------------- SYS = LRC
	CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_LRC);
		//------------------------- disable HRC
	CMU_Disable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
  
	//HIIPM:OFF
	GPIOA->HIIPM = 0x0000;
	GPIOB->HIIPM = 0x0000;
	GPIOC->HIIPM = 0x0000;
	GPIOD->HIIPM = 0x0000;
	//Mode:GPIO
	GPIOA->IOCFG = 0x0000;
	GPIOB->IOCFG = 0x0000;
	GPIOC->IOCFG = 0x0000;
	GPIOD->IOCFG = 0x0000;
	
	//function:input
	GPIOA->PTDIR = 0x0000;
	GPIOB->PTDIR = 0x0000;
	GPIOC->PTDIR = 0x0000;
	GPIOD->PTDIR = 0x0000;
	
	//PUPDN:PULLUP
	GPIOA->PUPDN = 0xFFFF0000;
	GPIOB->PUPDN = 0xFFFF0000;
	GPIOC->PUPDN = 0xFFFF0000;
	GPIOD->PUPDN = 0xFFFF0000;
	
	PMU->PMUCON &= ~(1<<4);
	SCB->SCR = 0x0000;
	__WFI();
}

/*
*********************************************************************************************************
*@function    EnterHold
*@brief       Enter hold mode.
*@param       None
*@retval      None
*********************************************************************************************************
*/
void Enter_Hold(void)
{	
	//------------------------- SYS = LRC
	CMU_Set_SYSCLK(CMU_SYSCLKCFG_CLKSEL_LRC);
	//	//------------------------- disable HRC
	CMU_Disable_CLKCTRL0(CMU_CLKCTRL0_HRCEN);
  
	//HIIPM:OFF
	GPIOA->HIIPM = 0x0000;
	GPIOB->HIIPM = 0x0000;
	GPIOC->HIIPM = 0x0000;
	GPIOD->HIIPM = 0x0000;
	//Mode:GPIO
	GPIOA->IOCFG = 0x0000;
	GPIOB->IOCFG = 0x0000;
	GPIOC->IOCFG = 0x0000;
	GPIOD->IOCFG = 0x0000;
	
	//function:input
	GPIOA->PTDIR = 0x0000;
	GPIOB->PTDIR = 0x0000;
	GPIOC->PTDIR = 0x0000;
	GPIOD->PTDIR = 0x0000;
	
	//PUPDN:PULLUP
	GPIOA->PUPDN = 0xFFFF0000;
	GPIOB->PUPDN = 0xFFFF0000;
	GPIOC->PUPDN = 0xFFFF0000;
	GPIOD->PUPDN = 0xFFFF0000;
  
	SCB->SCR = 0x0004;
	__WFI();
}

